/*************************************************************************
 * 
 * TheoTrade CONFIDENTIAL
 * __________________
 * 
 *  [2020] TheoTrade LLC 
 *  All Rights Reserved.
 * 
 * NOTICE:  All information contained herein is, and remains
 * the property of TheoTrade LLC and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to TheoTrade LLC
 * and its suppliers and may be covered by U.S. and Foreign Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from TheoTrade LLC.
 */

const predef = require("./tools/predef");
const p = require("./tools/plotting");


class TheoFutureLinesV2 {
    
    init() {
        const product = this.contractInfo.product;
        
        
        if (product.includes("NQ")) {
            this.range = 100;
            this.risk = [-48, -38, -28, -18, -8, 2, 12, 22, 32, 42];
            this.spot = [-44, -34, -23, -12, -4, 6, 18, 26, 36, 46];
            this.hundred = [-38, -12, 12, 33];
        } else if (product.includes("ES")) {
            this.range = 100;
            this.risk = [
                -48, -42, -38, -34, -29, -25.5, -21.5, -18, -12, -8, -2,
                2, 8, 11.5, 14.5, 18, 21.5, 25, 30, 34.5, 38, 42, 48];
            this.spot = [
                -45, -40, -36, -32, -27, -24, -20, -15, -10, -5, 
                0, 5, 10, 13, 16, 20, 23, 28, 32, 36, 40, 45, 50];
            this.hundred = [];
        } else if (product.includes("CL")) {
            this.range = 1;
            this.risk = [-0.9, -0.7, -0.5, -0.3, -0.1, 0.1, 0.3, 0.5, 0.7, 0.9];
            this.spot = [-0.8, -0.6, -0.4, -0.2,    0, 0.2, 0.4, 0.6, 0.8];
            this.hundred = [];
        } else {
            this.risk = [];
            this.spot = [];
            this.hundred = [];
        }
        this.highest = 0;
        this.lowest = 1000000;
        this.lower = 0;
        this.upper = 0;
        
        console.log(this.contractInfo);
        
    }
    
    map(d) {
        
        if (d.value() < this.lowest) {
            this.lowest = d.value();
            this.lower = Math.floor(this.lowest / this.range) * this.range;
        }
        
        if (d.value() > this.highest) {
            this.highest = d.value();
            this.upper = Math.ceil(this.highest / this.range) * this.range + this.range;
        }
        
        return {};
    }
    
}


function levels(canvas, instance, history) {
    const product = instance.contractInfo.product;
    const highest = instance.highest;
    const lowest = instance.lowest;
    const top = instance.upper;
    const bottom = instance.lower;
        
    const props = instance.props;
    const only100 = props["100Only"];
        
    if (highest > 0 &&
        history.data.length > 0) {
                
        // first point for line start position
        const first = history.get(0);
        const firstX = p.x.get(first);
            
        // last point for line end position
        const last = history.get(history.data.length - 1);
        const lastX = p.x.relative(p.x.get(last), 1000);
            
        const risk = instance.risk;
        const spot = instance.spot;
        const hundred = instance.hundred;
            
        const length = (top - bottom) / instance.range;
            
            
        for(let i=0; i<length; i++) {
            let range = i*instance.range;
            
            if (!only100 && product.includes("NQ")) {    
                // draw 33 risk
                canvas.drawLine(
                    p.offset(firstX, bottom + range + 33),
                    p.offset(lastX, bottom + range + 33),
                    {
                        color: props["33Color"],
                        width: props["33Width"],
                        opacity: props["33Op"]
                    }
                );
            
            
                // draw 25 risk
                canvas.drawLine(
                    p.offset(firstX, bottom + range + 25),
                    p.offset(lastX, bottom + range + 25),
                    {
                        color: props["25Color"],
                        width: props["25Width"],
                        opacity: props["25Op"]
                    }
                );    
            }
            
            if (product.includes("NQ")) {
                // draw 50 risk
                canvas.drawLine(
                    p.offset(firstX, bottom + range),
                    p.offset(lastX, bottom + range),
                    {
                        color: props["50Color"],
                        width: props["50Width"],
                        opacity: props["50Op"]
                    }
                );
                canvas.drawLine(
                    p.offset(firstX, bottom + 50 + range),
                    p.offset(lastX, bottom + 50 + range),
                    {
                        color: props["50Color"],
                        width: props["50Width"],
                        opacity: props["50Op"]
                    }
                );
            }
            
            if (!only100) {
                for(let j=0; j<risk.length; j++) {
                    let ry = bottom + risk[j] + range;
                    let sy = bottom + spot[j] + range;
                
                    // draw risk line
                    canvas.drawLine(
                        p.offset(firstX, ry),
                        p.offset(lastX, ry),
                        {
                            color: props["RiskColor"],
                            width: props["RiskWidth"],
                            opacity: props["RiskOp"]
                        }
                    );
                        
                        
                    // draw spot line
                    canvas.drawLine(
                        p.offset(firstX, sy),
                        p.offset(lastX, sy),
                        {
                            color: props["SpotColor"],
                            width: props["SpotWidth"],
                            opacity: props["SpotOp"]
                        }
                    );
                }    
            } else {
                for(let j=0; j<hundred.length; j++) {
                    let hy = bottom + hundred[j] + range;
                
                    // draw risk line
                    canvas.drawLine(
                        p.offset(firstX, hy),
                        p.offset(lastX, hy),
                        {
                            color: props["RiskColor"],
                            width: props["RiskWidth"],
                            opacity: props["RiskOp"]
                        }
                    );
                }
            }
        }
    }
}

module.exports = {
    name: "TheoFutureLinesV2",
    description: "TheoFutures NQ, ES, CL Lines V2",
    calculator: TheoFutureLinesV2,
    params: {
        "100Only": predef.paramSpecs.bool(false),
        "SpotColor": predef.paramSpecs.color("#FFD700"),
        "SpotWidth": {
            type: "number",
            def: 1.0,
            restrictions: {
                max: 5.0,
                min: 0.1,
                step: 0.1
            }
        },
        "SpotOp": {
            type: "number",
            def: 1.0,
            restrictions: {
                max: 1.0,
                min: 0.0,
                step: 0.1
            }
        },
        
        "RiskColor": predef.paramSpecs.color("cyan"),
        "RiskWidth": {
            type: "number",
            def: 0.5,
            restrictions: {
                max: 5.0,
                min: 0.1,
                step: 0.1
            }
        },
        "RiskOp": {
            type: "number",
            def: .8,
            restrictions: {
                max: 1.0,
                min: 0.0,
                step: 0.1
            }
        },
        
        "33Color": predef.paramSpecs.color("white"),
        "33Width": {
            type: "number",
            def: 1.0,
            restrictions: {
                max: 5.0,
                min: 0.1,
                step: 0.1
            }
        },
        "33Op": {
            type: "number",
            def: 1.0,
            restrictions: {
                max: 1.0,
                min: 0.0,
                step: 0.1
            }
        },
        
        "50Color": predef.paramSpecs.color("red"),
        "50Width": {
            type: "number",
            def: 1.0,
            restrictions: {
                max: 5.0,
                min: 0.1,
                step: 0.1
            }
        },
        "50Op": {
            type: "number",
            def: 0.8,
            restrictions: {
                max: 1.0,
                min: 0.0,
                step: 0.1
            }
        },
        
        "25Color": predef.paramSpecs.color("green"),
        "25Width": {
            type: "number",
            def: 1.0,
            restrictions: {
                max: 5.0,
                min: 0.1,
                step: 0.1
            }
        },
        "25Op": {
            type: "number",
            def: 1.0,
            restrictions: {
                max: 1.0,
                min: 0.0,
                step: 0.1
            }
        }
    },
    plotter: [
        predef.plotters.custom(levels)
    ],
    tags: ['TheoTrade'],
    schemeStyles: {}
    
};
