const lodash = require("lodash");

module.exports = {
    createPath() {
        let items = [];
        let makeCopyOnWrite = false;
        const copyOnWrite = () => {
            if (makeCopyOnWrite) {
                items = [].concat(items);
                makeCopyOnWrite = false;
            }
        };
        const path = {
            moveTo(x, y) {
                copyOnWrite();
                items.push({
                    t: "M",
                    x: lodash.clone(x),
                    y
                });
                return path;
            },
            lineTo(x, y) {
                copyOnWrite();
                items.push({
                    t: "L",
                    x: lodash.clone(x),
                    y
                });
                return path;
            },
            end() {
                makeCopyOnWrite = true;
                return {
                    t: "path",
                    items
                };
            }
        };
        return path;
    },

    createHeatmap(lowerBound, upperBound) {
        let items = [];
        let makeCopyOnWrite = false;
        const copyOnWrite = () => {
            if (makeCopyOnWrite) {
                items = [].concat(items);
                makeCopyOnWrite = false;
            }
        };
        const heatmap = {
            addColumn(x, cellColors) {
                copyOnWrite();
                items.push({
                    x,
                    c: cellColors
                });
            },
            end() {
                makeCopyOnWrite = true;
                return {
                    t: "heatmap",
                    lowerBound,
                    upperBound,
                    items
                };
            }
        };
        return heatmap;
    },

    x: {
        get(historyItem) {
            if (typeof historyItem.__dateIndex === "number")
                return historyItem.__dateIndex;
            return historyItem.date;
        },
        relative(x, dx) {
            return {
                x,
                dx
            };
        },
        between(x1, x2, k) {
            return {
                x1,
                x2,
                k
            };
        }
    },

    offset(x, y) {
        return {
            x,
            y
        };
    }
};
